# `@salesforce-ux/sds-styling-hooks`

## About

Global Styling hooks use CSS custom properties which make it easy to customize application-wide styling.

### Table of Contents

- [Installation](#installation)
  - [Add SDS Global Hooks in HTML](#add-sds-global-hooks-in-html)
  - [Add SDS Global Hooks in CSS](#add-sds-global-hooks-in-css)
  - [File Formats](#file-formats)
- [Usage Guidelines](#usage-guidelines)
  - [Color](#color)
    - [Using Color Relationships for UIs](#using-color-relationships-for-uis)
    - [Generic Palettes](#generic-palettes)
  - [Typography](#typography)
    - [Font Family](#font-family)
    - [Font Sizes](#font-sizes)
    - [Font Styles](#font-styles)

### Relevant links (external)

- Non-normative Specifications
  - [Global Styling Hooks](https://github.com/salesforce-ux/salesforce-design-system/blob/main/rfcs/text/0000-global-styling-hooks.md)
  - [Component Styling Hooks](https://github.com/salesforce-ux/salesforce-design-system/blob/main/rfcs/text/0000-component-styling-hooks.md)
- Styling Hooks Documentation
  - [Styling Hook Rules](https://github.com/salesforce-ux/salesforce-design-system/blob/main/docs/styling-hooks/rules.md)
  - [Styling Hook Naming Conventions](https://github.com/salesforce-ux/salesforce-design-system/blob/main/docs/styling-hooks/naming_conventions.md)
  - [How to use Styling Hooks in your Sub-system](https://github.com/salesforce-ux/salesforce-design-system/blob/main/docs/subsystems/how_to_style.md)

## Installation

Use [npm][npm] to install the [SDS Global Styling Hooks](https://www.npmjs.com/package/@salesforce-ux/sds-styling-hooks):

```bash
npm install --save-dev @salesforce-ux/sds-styling-hooks
```

### Add SDS Global Hooks in HTML

You can use the HTML `<link>` to link to SDS Global Styling Hooks as an external resource.

```html
<html>
  <head>
    <link rel="stylesheet" href="/node_modules/@salesforce-ux/sds-styling-hooks/dist/hooks.custom-props.css">
    <!-- Your application's stylesheets go below -->
    <link rel="stylesheet" href="...">
  </head>
  <body>
    <!-- Your application -->
  </body>
</html>
```

### Add SDS Global Hooks in CSS

You can use CSS `@imports` to pull in SDS Global Styling Hooks.

```css
:root {
  @import "@salesforce-ux/sds-styling-hooks/dist/hooks.custom-props.css";
}

body {
  /* Directly reference SDS Styling Hook */
  background: var(--sds-g-color-neutral-base-1);
}
```

### File Formats

SDS Styling Hooks have 6 different file formats for consumption. All files are located in `/dist/`.

#### JSON (`*.json`)

```json
{
  "PROP_NAME": "PROP_VALUE"
}
```

#### Raw JSON (`*.raw.json`)

```js
{
  "props": {
    "PROP_NAME": {
      "value": "PROP_VALUE",
      "type": "PROP_TYPE",
      "category": "PROP_CATEGORY",
      /* additional metadata */
    }
  }
}
```

#### Common JS (`*.js`)

```js
module.exports = {
  propName: "PROP_VALUE"
};
```

#### ES Module (`*.js`)

```js
export const propName = "PROP_VALUE";
```

#### Custom Properties scoped to `:root` (`*.css`)

```css
:root {
  --prop-name: PROP_VALUE;
}
```

#### Custom Property Registration using [CSS Properties and Values API](https://drafts.css-houdini.org/css-properties-values-api-1/) (`*.js`)

> **Experimental**
>
> This is an experimental technology
>
> Check the [Browser compatibility table](https://caniuse.com/mdn-api_css_registerproperty) carefully before using this in production.

For more information, see [CSS Properties and Values API Level 1](https://drafts.css-houdini.org/css-properties-values-api-1/).

```js
if (CSS.registerProperty) {
  CSS.registerProperty({
    name: "--prop-name",
    syntax: "PROP_SYNTAX",
    inherits: false,
    initialValue: "PROP_VALUE",
  });
}
```

## Usage Guidelines

### Color

The Salesforce Design System comes with a robust color system and relationships to ensure our UIs meet [WCAG 2.1 contrast ratios of 4.5:1](https://www.w3.org/TR/WCAG21/#contrast-minimum). The semantic relationships allow for easy theme switching to handle features such as dark mode or high contrast mode.

Additionally, we generate sets of vibrant color palettes to use as accents in your designs. These values are generated in the [HCL color space](https://www.boronine.com/2012/03/26/Color-Spaces-for-Human-Beings/), which are designed to accord with human perception of color.

#### Using Color Relationships for UIs

[View UI color relationships →](https://design-system-services.herokuapp.com/?path=/story/sds-styling-hooks-colors--ui-colors)

The Salesforce Design System offers a set of neutral and brand color relationships. These two sets are intended to be used on your UI elements to define properties such as backgrounds, borders and text.

Each set, neutral and brand, have two groupings called **base** and **inverse**. The base group defaults to "light" UI elements. The inverse color group are the opposite values of the base group, these default to "dark" UI elements.

**Neutral**
```css
--sds-g-color-neutral-base-1
--sds-g-color-neutral-base-2
--sds-g-color-neutral-base-3
--sds-g-color-neutral-base-4

--sds-g-color-neutral-base-contrast-1
--sds-g-color-neutral-base-contrast-2
--sds-g-color-neutral-base-contrast-3
--sds-g-color-neutral-base-contrast-4
```

**Brand**
```css
--sds-g-color-brand-base-1
--sds-g-color-brand-base-2
--sds-g-color-brand-base-3
--sds-g-color-brand-base-4

--sds-g-color-brand-base-contrast-1
--sds-g-color-brand-base-contrast-2
--sds-g-color-brand-base-contrast-3
--sds-g-color-brand-base-contrast-4
```

Each group contains 8 color values; 4 of which are the default color values for that group and 4 are the **contrast** variants. These hold the color relationships in the set.

![An image showing the color mappings for UI color relationships](images/ui-color-relationships.jpg)

**Golden Rule for Relationships**

Regardless of the set, neutral or brand, there is a golden rule to ensure the color relationships are retained;

1. When you use a base or inverse value as your background color, a contrast value in that group is required to be used on top of that background, i.e., if `base-2` is the background, then `base-contrast-2` is your text color. If `inverse-1` is the background, then `inverse-contrast-1` is your text color.
2. The second requirement is to use the correct number value. You can only use a contrast value where the number is equal to or greater than the background color number value, i.e. if `base-2` is the background, then `base-contrast-2`, `base-contrast-3`, `base-contrast-4` as your text will always ensure a 4.5:1 contrast ratio.

![An image showing the neutral color UI relationships](images/neutral-group.png)

#### Generic Palettes

[View all color palettes →](https://design-system-services.herokuapp.com/?path=/story/sds-styling-hooks-colors--palettes)

The Salesforce Design System provide generic color palettes to use in your designs. These values are generated in the [HCL color space](https://www.boronine.com/2012/03/26/Color-Spaces-for-Human-Beings/), which are designed to accord with human perception of color.

> **Important usage note**
>
> If you use a color from these palettes on a UI element, you are responsible to ensure proper contrast ratio between the background and text. Additionally, the color cannot be programmically alter if your application requires different themes such as dark mode or high contrast mode.

![An image showing the indigo color UI relationships](images/indigo-palette.png)

### Typography

#### Font Family

[View Font Families →](https://design-system-services.herokuapp.com/?path=/story/sds-styling-hooks-typography--font-family)

The Salesforce Design System defaults to use system fonts, this improves performance and legibility on the content based on the device being used. We provide two styling hooks for font families, one of the body text content and one for the heading text of your document.

**Body Text Content**
```css
--sds-g-font-family
```

**Heading Text Content**
```css
--sds-s-heading-font-family
```

**Font Stack**
```css
font-family: -apple-system, BlinkMacSystemFont, Segoe UI, Roboto, 'Helvetica Neue', Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol';
```

#### Font Sizes

[View Font Sizes →](https://design-system-services.herokuapp.com/?path=/story/sds-styling-hooks-typography--font-size)

The Salesforce Design System offers a robust palette of font sizes. Our font sizes are independent of semantics so they are intended to be used for visual hierarchy. The sizes are based off a [major second scale](https://www.modularscale.com/?1&rem&1.125), 8:9 scale ratio with the base font size starting at 1rem or 16px. The scale offers 10 font sizes larger than the base and 4 font sizes smaller than the base.

![An image showing the font scale](images/font-scale.png)

#### Font Styles

##### Weights

[View Font Weights →](https://design-system-services.herokuapp.com/?path=/story/sds-styling-hooks-typography--font-weight)

[npm]: https://www.npmjs.com/